// Constants
const SECURE = Components.interfaces.nsIWebProgressListener.STATE_IS_SECURE |
               Components.interfaces.nsIWebProgressListener.STATE_SECURE_HIGH;
const NO_PETNAME = "untrusted";
const INBOX = "Petname Inbox";

// State
var key = null;
const listener = {
    onStateChange:function(aProgress,aRequest,aFlag,aStatus) {},
    onProgressChange:function(a,b,c,d,e,f) {},
    onLocationChange:function(a,b,c) {},
    onStatusChange:function(a,b,c,d) {},
    onSecurityChange:function(progress, request, state) {

        key = null;
        const petname = document.getElementById("petname-text");
        if (state == SECURE) {

            // Get the entity certificate.
            const sui = window.getBrowser().securityUI;
            const ssl = sui.QueryInterface(
                Components.interfaces.nsISSLStatusProvider);
            const status = ssl.SSLStatus;
            const ssl_status = status.QueryInterface(
                Components.interfaces.nsISSLStatus);
            const ee = ssl_status.serverCert;

            // Get the root CA certificate.
            var ca = ee;
            for (i = ee.getChain().length; --i != 0;) {
                ca = ca.issuer;
            }

            // Populate the petname box.
            key = ca.sha1Fingerprint + "'s \"" + ee.organization + "\"";
            const rdf = Components.classes["@mozilla.org/rdf/rdf-service;1"].
                getService(Components.interfaces.nsIRDFService);
            const bookmarks = Components.
                classes["@mozilla.org/rdf/datasource;1?name=bookmarks"].
                    getService(Components.interfaces.nsIRDFDataSource);
            const bookmark = bookmarks.GetSource(
                rdf.GetResource("http://home.netscape.com/NC-rdf#Description"),
                rdf.GetLiteral(key), true);
            if (bookmark != null) {
                // Found an existing trust relationship.
                const title = bookmarks.GetTarget(bookmark,
                    rdf.GetResource("http://home.netscape.com/NC-rdf#Name"),
                    true);
                if (title instanceof Components.interfaces.nsIRDFLiteral) {
                    petname.value = title.Value;
                    petname.setAttribute("class", "PetnameTool-populated");
                }
            } else {
                // No existing trust relationship.
                petname.value = NO_PETNAME;
                petname.setAttribute("class", "PetnameTool-enabled");
            }
            petname.disabled = false;
        } else {
            petname.value = NO_PETNAME;
            petname.setAttribute("class", "PetnameTool-disabled");
            petname.disabled = true;
        }
    },

    QueryInterface: function(iid) {
        if (iid.equals(Components.interfaces.nsIWebProgressListener) ||
            iid.equals(Components.interfaces.nsISupportsWeakReference) ||
            iid.equals(Components.interfaces.nsISupports)) {
            return this;
        }
        throw Components.results.NS_ERROR_NO_INTERFACE;
    }
}

// Initialization
window.addEventListener("load", function(evt) {
    setTimeout(PetnameTool_Load, 0, evt); }, false);
window.addEventListener("unload", PetnameTool_Unload, false);

function
PetnameTool_Load() {
    window.getBrowser().addProgressListener(listener,
        Components.interfaces.nsIWebProgress.NOTIFY_SECURITY);
}

function
PetnameTool_Unload() {
    window.getBrowser().removeProgressListener(listener);
}

// Operations

function
PetnameTool_Assign(event) {
    if (null == key) {
        alert("No identifiable site!");
    } else {
        // Check for an existing bookmark.
        const rdf = Components.classes["@mozilla.org/rdf/rdf-service;1"].
            getService(Components.interfaces.nsIRDFService);
        const bookmarks = Components.
            classes["@mozilla.org/rdf/datasource;1?name=bookmarks"].
                getService(Components.interfaces.nsIRDFDataSource);
        const bookmark = bookmarks.GetSource(
            rdf.GetResource("http://home.netscape.com/NC-rdf#Description"),
            rdf.GetLiteral(key), true);
        const petname = document.getElementById("petname-text");
        if (petname.value == NO_PETNAME || petname.value == "") {
            // Delete the existing bookmark.
            if (bookmark != null) {

                // Wipe the key, since deletion of the bookmark doesn't
                // seem to take.
                bookmarks.Change(bookmark, rdf.GetResource(
                        "http://home.netscape.com/NC-rdf#Description"),
                    rdf.GetLiteral(key), rdf.GetLiteral(""));

                // Unhook the bookmark from the bookmark list.
                const arcsIn = bookmarks.ArcLabelsIn(bookmark);
                while (arcsIn.hasMoreElements()) {
                    const arcIn = arcsIn.getNext();
                    const srcs = bookmarks.GetSources(arcIn, bookmark, true);
                    while (srcs.hasMoreElements()) {
                        bookmarks.Unassert(srcs.getNext(), arcIn, bookmark);
                    }
                }

                // Initialize the petname tool.
                petname.value = NO_PETNAME;
                petname.setAttribute("class", "PetnameTool-enabled");
            }
        } else {
            if (bookmark != null) {
                // Rename the existing bookmark.
                const title = bookmarks.GetTarget(bookmark,
                    rdf.GetResource("http://home.netscape.com/NC-rdf#Name"),
                    true);
                bookmarks.Change(bookmark,
                    rdf.GetResource("http://home.netscape.com/NC-rdf#Name"),
                    title, rdf.GetLiteral(petname.value));
            } else {
                // Create a new bookmark.
                const bookmark_maker = Components.
                    classes["@mozilla.org/browser/bookmarks-service;1"].
                        getService(Components.interfaces.nsIBookmarksService);
                inbox = bookmarks.GetSource(rdf.GetResource(
                    "http://home.netscape.com/NC-rdf#Description"),
                    rdf.GetLiteral(INBOX), true);
                if (inbox == null) {
                    inbox = bookmark_maker.getBookmarksToolbarFolder();
                }
                const page = window.getBrowser().webNavigation;
                bookmark_maker.createBookmarkInContainer(petname.value,
                    page.currentURI.spec, "", key, page.document.characterSet,
                    "", inbox, 0);
                petname.setAttribute("class", "PetnameTool-populated");
            }
        }
    }
    event.preventBubble();
}
